#!/bin/env python 
##############################################################################
# use ftp to copy (download) all files from a remote site and directory
# to a directory on the local machine; this version works the same, but has
# been refactored to wrap up its code in functions that can be reused by the
# uploader, and possibly other programs in the future - else code redundancy,
# which may make the two diverge over time, and can double maintenance costs.
##############################################################################

import os, sys, ftplib
from getpass   import getpass
from mimetypes import guess_type, add_type

defaultSite = 'home.rmi.net'
defaultRdir = '.'
defaultUser = 'lutz'

def configTransfer(site=defaultSite, rdir=defaultRdir, user=defaultUser):
    """
    get upload or download parameters
    uses a class due to the large number
    """
    class cf: pass
    cf.nonpassive = False                 # passive ftp on by default in 2.1+
    cf.remotesite = site                  # transfer to/from this site 
    cf.remotedir  = rdir                  # and this dir ('.' means acct root)
    cf.remoteuser = user
    cf.localdir   = (len(sys.argv) > 1 and sys.argv[1]) or '.'
    cf.cleanall   = raw_input('Clean target directory first? ')[:1] in ['y','Y']
    cf.remotepass = getpass(
                    'Password for %s on %s:' % (cf.remoteuser, cf.remotesite))
    return cf

def isTextKind(remotename, trace=True):
    """
    use mimetype to guess if filename means text or binary
    for 'f.html,   guess is ('text/html', None): text
    for 'f.jpeg'   guess is ('image/jpeg', None): binary
    for 'f.txt.gz' guess is ('text/plain', 'gzip'): binary
    for unknowns,  guess may be (None, None): binary
    mimetype can also guess name from type: see PyMailGUI
    """
    add_type('text/x-python-win', '.pyw')                       # not in tables
    mimetype, encoding = guess_type(remotename, strict=False)   # allow extras
    mimetype  = mimetype or '?/?'                               # type unknown?
    maintype  = mimetype.split('/')[0]                          # get first part
    if trace: print maintype, encoding or ''
    return maintype == 'text' and encoding == None              # not compressed

def connectFtp(cf):
    print 'connecting...'
    connection = ftplib.FTP(cf.remotesite)           # connect to ftp site
    connection.login(cf.remoteuser, cf.remotepass)   # login as user/password
    connection.cwd(cf.remotedir)                     # cd to directory to xfer
    if cf.nonpassive:                                # force active mode ftp
        connection.set_pasv(False)                   # most servers do passive
    return connection

def cleanLocals(cf):
    """
    try to delete all locals files first to remove garbage
    """
    if cf.cleanall:
        for localname in os.listdir(cf.localdir):    # local dirlisting
            try:                                     # local file delete
                print 'deleting local', localname
                os.remove(os.path.join(cf.localdir, localname))
            except:
                print 'cannot delete local', localname

def downloadAll(cf, connection):
    """
    download all files from remote site/dir per cf config
    ftp nlst() gives files list, dir() gives full details
    """     
    remotefiles = connection.nlst()                  # nlst is remote listing
    for remotename in remotefiles:
        localpath = os.path.join(cf.localdir, remotename) 
        print 'downloading', remotename, 'to', localpath, 'as',
        if isTextKind(remotename):
            # use text mode xfer
            localfile = open(localpath, 'w')
            def callback(line): localfile.write(line + '\n')
            connection.retrlines('RETR ' + remotename, callback)
        else:
            # use binary mode xfer
            localfile = open(localpath, 'wb')
            connection.retrbinary('RETR ' + remotename, localfile.write)
        localfile.close()
    connection.quit()
    print 'Done:', len(remotefiles), 'files downloaded.'

if __name__ == '__main__':
    cf = configTransfer()
    conn = connectFtp(cf)
    cleanLocals(cf)          # don't delete if can't connect
    downloadAll(cf, conn)
